/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.jmx.mx4j;

// JMX includes
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;

// MX4J includes
import mx4j.adaptor.rmi.jrmp.JRMPAdaptorMBean;
import mx4j.util.StandardMBeanProxy;

// JOnAS includes
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.jonas.jmx.AbsJmxServiceImpl;
import org.objectweb.jonas.jmx.JonasObjectName;

// Monolog
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;


/**
 * JMX Service implementation with mx4j.
 * Provides the specific doStart() method.
 * @author Adriana Danes

 */

public class Mx4jJmxServiceImpl extends AbsJmxServiceImpl {

    private JRMPAdaptorMBean jrmpAdaptorMbean = null; // facilitates the manipulation of the JRMP adaptor


    /**
     * Start the Service.
     * Create the JRMP adapter MBean.
     */
    public void doStart() throws ServiceException {       

	try {
	    // Create a MX4J JRMP adaptor : a RMI adaptor provided by MX4J
	    // This is an MBean, so it has an ObjectName and is to be registered 
	    // in the MBeanServer

	    // create the ObjectName
	    ObjectName adaptor = JonasObjectName.jrmpAdaptor();

	    // Create the MBean 
	    //    the third parameter (the loaderName) beeing null, the class 
	    //    loader is the one that loaded the MBeanServer
	    jmxServer.createMBean("mx4j.adaptor.rmi.jrmp.JRMPAdaptor", adaptor, null);

	    // Create a StandardMBeanProxy 
	    // This is a proxy for the MBean with the specified name (adaptor) and
	    // living in the specified MBeanServer (jmxServer).
	    jrmpAdaptorMbean = (JRMPAdaptorMBean)StandardMBeanProxy.create(
							     JRMPAdaptorMBean.class, 
							     jmxServer, 
							     adaptor);
	    // Set the JNDI name with which will be registered in the registry
	    //String jndiName = "jrmp"; - this is the default name used by MC4J
	    String jndiName = "RMIConnector_"+jonasName;
	    jrmpAdaptorMbean.setJNDIName(jndiName);

	    // Register the JRMP adaptor in JNDI and start it
	    jrmpAdaptorMbean.start();
	} catch (Exception e) {
	    logger.log(BasicLevel.DEBUG, "Cannot start JMX service "+e);
	    throw new ServiceException("Cannot start JMX service", e);
	}
    }

    /**
     * Stop this service
     */
    public void doStop() {
	try {
	    // stop the adapter, so it does not accept incoming calls anymore
	    jrmpAdaptorMbean.stop();
	} catch (Exception e) {
	    logger.log(BasicLevel.ERROR, "Cannot Unbind Jmx RMI Connector" + e);
	}
	// Remove internal references to the MBeanServer. 
	MBeanServerFactory.releaseMBeanServer(jmxServer); 
	logger.log(BasicLevel.DEBUG, "JMX Service stopped");
    }

    /**
     * @return The reference of the MBean server
     */
    public MBeanServer getJmxServer() {
	return this.jmxServer;
    }
}
