/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Adriana Danes
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin;

import java.util.Set;
import java.util.Properties;
import javax.naming.Context;
import javax.management.ObjectName;
import javax.management.MBeanInfo;

/**
 * Wraper class. Apply the management operations on the ManagementRep instance. 
 * The ManagementRep is an instance of one of the provided implementations. Curretly, is
 * an instance of ManagementRepImpl, if the Jonas JmxService is based on sun ri, or
 * an instance of Mx4jManagementRepImpl, if the Jonas JmxService is based on the mx4j jmx implementation.
 */
public class JonasManagementRepr {

    static ManagementRepr rep = ManagementReprFactory.getManagementRepr();

    /**
     * Test if rep null. This means that the ManagementRepFactory did not managed to instantiate the ManagementRep class.
     */
    public static boolean reprNull() {
	if (rep == null)
	    return true;
	else
	    return false;
    }
    /**
     * @return True if the MBean is already registered in the MBean server, false otherwise or if an exception is catched.
     */
    public static boolean isRegistered(ObjectName on) {
	return rep.isRegistered(on);
    }

    /**
     * @param on The ObjectName of the MBean from which the attribute is to be retrieved.
     * @param attribute A String specifying the name of the attribute to be retrieve.
     * @return The value of the attribute.
     */
    public static Object getAttribute(ObjectName on, String attribute) 
	throws ManagementException 
    {
	return rep.getAttribute(on, attribute); 
    }

    /**
     * @param on The ObjectName of the MBean within which the attribute is to be set.
     * @param attribute A String specifying the name of the attribute to be retrieve.
     * @param value The value to set to the attribute.
     */
    public static void setAttribute(ObjectName on, String attribute, Object value) 
	throws ManagementException 
    {
	rep.setAttribute(on, attribute, value);
    }

    /**
     * @param on
     */
    public static Object invoke(ObjectName on, String operation, Object[] param, String[] signature) 
	throws ManagementException 
    {
	return rep.invoke(on, operation, param, signature);
    }

    /**
     * @return A set containing the ObjectNames for the MBeans selected.
     */
    public static java.util.Set queryNames(ObjectName on) throws ManagementException 
    {
	return rep.queryNames(on);
    }

    /**
     * @return An instance of MBeanInfo allowing the retrieval of all 
     * attributes and operations of this MBean.
     */    
    public static MBeanInfo getMBeanInfo(ObjectName name) throws ManagementException {
	return (MBeanInfo)rep.getMBeanInfo(name);
    }

    /**
     * @return Context the current application context.
     */
    public static Context getContext() throws javax.naming.NamingException {
	return rep.getContext();
    }    
    /**
     * @return String the name of the current RMI connector.   
     * Return null if no RMI connector is available.
     */
    public static String getCurrentRMIConnectorName() {
	return rep.getCurrentRMIConnectorName();
    }
    /**
     * Set the currentRMIConnectorName to the specified value
     */
    public static void setCurrentRMIConnectorName(String name) throws Exception {
	rep.setCurrentRMIConnectorName(name);
    }
    /**
     * Set the currentRMIConnectorName to null
     */
    public static void resetCurrentRMIConnectorName() {
	rep.resetCurrentRMIConnectorName();
    }    
    /**
     * @return Set a set containning all RMI connector names available in the current context.
     */
    public static Set getRMIConnectorsNames() throws javax.naming.NamingException {
	return rep.getRMIConnectorsNames();
    }
    /**
     * @return String the value of the PROVIDER_URL property in the current context.
     */
    public static String getJonasNamingServiceURL() { 
	return rep.getJonasNamingServiceURL();
    }    
    /**
     * Sets  the PROVIDER_URL property to the specified value. 
     */
    public static void setJonasNamingServiceURL(String url) throws javax.naming.NamingException {
	rep.setJonasNamingServiceURL(url);
    }

    /**
     * Create a new naming context based on the given env. properties
     * @param env properties to create a new naming context
     */
    public static void setNamingEnvCtx(Properties env) throws javax.naming.NamingException { 
	rep.setNamingEnvCtx(env);
    }   
}
