/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Adriana Danes
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.jmx.vjadmin;

import java.util.Properties;
import java.io.InputStream;

// JOnAS Log
import org.objectweb.jonas.common.Log;

// Monolog
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

public class ManagementReprFactory {

    static String defaultName = "org.objectweb.jonas.jmx.vjadmin.Mx4jManagementReprImpl";
    static String propName = "Jadmin.ConnectorFactory.class";
    static String propFileName = "Jadmin.properties";

    static private Logger logger = null;

    public static ManagementRepr getManagementRepr() {
	logger = Log.getLogger("org.objectweb.jonas.jmx.vjadmin");
	
	ManagementRepr representative = null;

	ClassLoader classLoader = ManagementReprFactory.class.getClassLoader();

	Properties props = new Properties();
	InputStream input = null;
	try {
	    //input = Thread.currentThread().getContextClassLoader().getResourceAsStream(propFileName);
	    input = classLoader.getResourceAsStream(propFileName);
	    props.load(input);
	} catch (Exception e) {
	    logger.log(BasicLevel.DEBUG, "ManagementReprFactory , can't load Jadmin properties : " + e.toString());
	}
       
	String managemntRepClassName = props.getProperty(propName);
	if (managemntRepClassName == null) {
	    managemntRepClassName = defaultName;
	    logger.log(BasicLevel.DEBUG, "ManagementReprFactory , the ManagementRep class to load is " + managemntRepClassName);
	}
	try {
	    Class managemntRepClass = classLoader.loadClass(managemntRepClassName);
	    logger.log(BasicLevel.DEBUG, "ManagementReprFactory , managemntRep class loaded");
	    representative = (ManagementRepr)managemntRepClass.newInstance();
	    logger.log(BasicLevel.DEBUG, "ManagementReprFactory , managemntRep created");
	} catch (Exception e) {
	    logger.log(BasicLevel.DEBUG, "ManagementReprFactory exception : " + e.toString());
	}
	return representative;
    }
}
