/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Adriana Danes
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.jmx.vjadmin;

import java.util.Set;
import java.util.Properties;

// JNDI
import javax.naming.Context;
import javax.naming.NamingException;

// JMX
import javax.management.ObjectName;
import javax.management.MBeanInfo;
import javax.management.Attribute;
import javax.management.MBeanException;
import javax.management.InstanceNotFoundException;
import javax.management.ReflectionException;
import javax.management.RuntimeMBeanException;
import javax.management.RuntimeOperationsException;
import javax.management.RuntimeErrorException;

// JOnAS Log
import org.objectweb.jonas.common.Log;

// Monolog
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

public class Mx4jManagementReprImpl implements ManagementRepr {

    static private Logger logger = null;

    protected Mx4jManagementReprImpl() {
	logger = Log.getLogger("objectweb.org.jonas.jmx.vjadmin");
	logger.log(BasicLevel.DEBUG, "Management Representativ created for work with mx4j jmx");
    }

    /**
     * @return True if the MBean is already registered in the MBean server, false otherwise or if an exception is catched.
     */
    public boolean isRegistered(ObjectName on) {
	try {
	    return Mx4jConnectorFactory.getRMIConnector().getRemoteMBeanServer().isRegistered(on);
	} catch (Exception e) {
	    return false;
	}	
    }  
    /**
     * @param on The ObjectName of the MBean from which the attribute is to be retrieved.
     * @param attribute A String specifying the name of the attribute to be retrieve.
     * @return The value of the attribute.
     */
    public   Object getAttribute(ObjectName on, String attribute) 
	throws ManagementException 
    {
	try {
	    return Mx4jConnectorFactory.getRMIConnector().getRemoteMBeanServer().getAttribute(on, attribute);
	} catch (Exception e) {
	    throw new ManagementException("Error while getting attribute "
					  +attribute+": "
					  +e.getClass().getName(), e);
	}
    } 
  /**
     * @param on The ObjectName of the MBean within which the attribute is to be set.
     * @param attribute A String specifying the name of the attribute to be retrieve.
     * @param value The value to set to the attribute.
     */
    public   void setAttribute(ObjectName on, String attribute, Object value) 
	throws ManagementException 
    {
	Logger logger = Log.getLogger("org.objectweb.jonas.jmx.vjadmin");
	logger.log(BasicLevel.DEBUG, "Set Attribute called, on " + on.toString() + " to change attribute " + attribute + " value to " + (String)value.toString());
	try {
	    Mx4jConnectorFactory.getRMIConnector().getRemoteMBeanServer().setAttribute(on, new Attribute(attribute, value));
	} catch (Exception e) {
	    throw new ManagementException("Error while setting attribute "
					  +attribute+": "
					  +e.getClass().getName(), e);
	}
    } 
    /**
     * @param on
     */
    public   Object invoke(ObjectName on, String operation, Object[] param, String[] signature) 
	throws ManagementException 
    {
	Logger logger = Log.getLogger("org.objectweb.jonas.jmx.vjadmin");
	try {
	    return Mx4jConnectorFactory.getRMIConnector().getRemoteMBeanServer().invoke(on, operation, param, signature);
	} catch (Exception e) {
	    logger.log(BasicLevel.DEBUG, "EXCEPTION when calling invoke. Exception class: " + e.getClass().getName());
	    String message = null;
	    Throwable exc = null;
	    if (e instanceof InstanceNotFoundException) {
		logger.log(BasicLevel.DEBUG, "--------- InstanceNotFoundException -------------");
		message = "InstanceNotFoundException when trying to use MBean " + on.toString();
	    } else if (e instanceof MBeanException) {
		Exception targetExc = ((MBeanException) e).getTargetException();
		logger.log(BasicLevel.DEBUG, "-------[ MBeanException ]-------");
		logger.log(BasicLevel.DEBUG, "-------[ Embedded Exception : ]-------");
		logger.log(BasicLevel.DEBUG, "-------[ " + targetExc.toString() + " ]-------");
		message = "Exception while invoking operation " + operation + " on MBean " + on.toString();
		exc = targetExc;		
	    } else if (e instanceof ReflectionException) {
		Exception targetExc = ((ReflectionException) e).getTargetException();		
		logger.log(BasicLevel.DEBUG, "-------[ ReflectionException ]-------");
		logger.log(BasicLevel.DEBUG, "-------[ Embedded Exception : ]-------");
		logger.log(BasicLevel.DEBUG, "-------[ " + targetExc.toString() + " ]-------");
		message = "ReflectionException while invoking operation " + operation + " on MBean " + on.toString();
		exc = targetExc;
	    } else if (e instanceof RuntimeMBeanException) {
		Exception targetExc = ((RuntimeMBeanException) e).getTargetException();		
		logger.log(BasicLevel.DEBUG, "-------[ RuntimeMBeanException ]-------");
		logger.log(BasicLevel.DEBUG, "-------[ Embedded Exception : ]-------");
		logger.log(BasicLevel.DEBUG, "-------[ " + targetExc.toString() + " ]-------");
		message = "RuntimeMBeanException while invoking operation " + operation + " on MBean " + on.toString();
		exc = targetExc;
	    } else if (e instanceof RuntimeOperationsException) {
		Exception targetExc = ((RuntimeOperationsException) e).getTargetException();		
		logger.log(BasicLevel.DEBUG, "-------[  RuntimeOperationException]-------");
		logger.log(BasicLevel.DEBUG, "-------[ Embedded Exception : ]-------");
		logger.log(BasicLevel.DEBUG, "-------[ " + targetExc.toString() + " ]-------");
		message = "RuntimeOperationException while invoking operation " + operation + " on MBean " + on.toString();
		exc = targetExc;
	    } else if (e instanceof RuntimeErrorException) {
		Error targetError = ((RuntimeErrorException) e).getTargetError();
		logger.log(BasicLevel.DEBUG, "-------[  RuntimeErrorException ]-------");
		logger.log(BasicLevel.DEBUG, "-------[ Embedded error : ]-------");
		logger.log(BasicLevel.DEBUG, "-------[ " + targetError.toString() + " ]-------");
		message = "RuntimeErrorException while invoking operation " + operation + " on MBean " + on.toString();
	    } else {
		// This situation is not possible cf. to the JMX specification
		message = "Unexpexted exception while invoking operation " + operation + " on MBean " + on.toString();
	    }
	    throw new ManagementException(message, exc);
	}
    }

    /**
     * @return A set containing the ObjectNames for the MBeans selected.
     */
    public   java.util.Set queryNames(ObjectName on) throws ManagementException 
    {
	try {
	    return (java.util.Set)Mx4jConnectorFactory.getRMIConnector().getRemoteMBeanServer().queryNames(on, null);
	} catch (Exception e) {
	    throw new ManagementException("Error while getting MBeans names: "
					  +e.getClass().getName(), e);
	}
    }

    /**
     * @return An instance of MBeanInfo allowing the retrieval of all 
     * attributes and operations of this MBean.
     */    
    public   MBeanInfo getMBeanInfo(ObjectName name) throws ManagementException {
	try {
	    return (MBeanInfo)Mx4jConnectorFactory.getRMIConnector().getRemoteMBeanServer().getMBeanInfo(name);
	} catch (Exception e) {
	    throw new ManagementException("Error while getting MBean info: " + e.getClass().getName(), e);
	}
    }
    /**
     * @return Context the current application context, create an initial context 
     * if there is no current context.
     */
    public   Context getContext() throws javax.naming.NamingException {
	return Mx4jConnectorFactory.getContext();
    }     
    /**
     * @return String the name of the current RMI connector.   
     * Return null if no RMI connector is available.
     */
    public   String getCurrentRMIConnectorName() {
	return Mx4jConnectorFactory.getCurrentRMIConnectorName();
    }
    /**
     * Set the currentRMIConnectorName to the specified value
     */
    public   void setCurrentRMIConnectorName(String name) throws Exception {
	Mx4jConnectorFactory.setCurrentRMIConnectorName(name);
    }
    /**
     * Set the currentRMIConnectorName to null
     */
    public   void resetCurrentRMIConnectorName() {
	Mx4jConnectorFactory.resetCurrentRMIConnectorName();
    }    
    /**
     * @return Set a set containning all RMI connector names available in the current context.
     */
    public   Set getRMIConnectorsNames() throws javax.naming.NamingException {
	return Mx4jConnectorFactory.getRMIConnectorsNames();
    }
    /**
     * @return String the value of the PROVIDER_URL property in the current context.
     */
    public   String getJonasNamingServiceURL() { 
	return Mx4jConnectorFactory.getJonasNamingServiceURL();
    }    
    /**
     * Sets  the PROVIDER_URL property to the specified value. 
     */
    public   void setJonasNamingServiceURL(String url) throws javax.naming.NamingException {
	Mx4jConnectorFactory.setJonasNamingServiceURL(url);
    } 

    /**
     * Create a new naming context based on the given env. properties
     * @param env properties to create a new naming context
     */
    public void setNamingEnvCtx(Properties env) throws javax.naming.NamingException {  
	Mx4jConnectorFactory.setNamingEnvCtx(env);
    }
}
