/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin.servlet;

// servlet imports
import javax.servlet.*;
import javax.servlet.http.*;

// standard imports
import java.util.Date;
import java.util.Properties;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.servlet.VelocityServlet;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;
// JMX
import javax.management.ObjectName;
import javax.management.MalformedObjectNameException;

// Jadmin imports
import org.objectweb.jonas.jmx.vjadmin.JonasManagementRepr;
import org.objectweb.jonas.jmx.vjadmin.ManagementException;

// monolog imports
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

// JOnAS log imports
import org.objectweb.jonas.common.Log;

public class MdbServlet extends EjbServlet {

    /**
     *
     *
     *  @param ctx a Velocity Context object to be filled with
     *             data.  Will be used for rendering this 
     *             template
     *  @return Template to be used for request
     */   

    public Template handleRequest( HttpServletRequest request, 
				   HttpServletResponse response, 
				   Context context ) { 

	// Exception treatement
	String alertException = null;

 	// Get Date
	String date = java.text.DateFormat.getInstance().format(new Date());
	context.put("date", date);

	// Get request parameters
	String fileName = null;
	String ejbName = null;
	try {
	    for (Enumeration e = request.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("fileName")) {
		    fileName = request.getParameter("fileName");
		    context.put("fileName", fileName);
		} else if (paramName.equals("ejbName")) {
		    ejbName = request.getParameter("ejbName");
		    context.put("ejbName", ejbName);
		}
	    }
	} catch (ManagementException e) {
	    alertException = alert(e.getMessage());
	    context.put("alertException", alertException);
	}

	// Get the template to be returned.
	Template template = null;
	try {
	    template = getTemplate("MDB.vm"); 
        } catch( Exception e) {
            logger.log(BasicLevel.ERROR, "Error " + e);
	    return null;
        }

	// Get the container's name
	String cont_error = "";
	try {
	    String contName = (String)JonasManagementRepr.getAttribute(JonasObjectName.container(fileName), "ContainerName");
	    context.put("contName", contName);
	} catch (Exception e) {
	    cont_error = "Error while getting attribute:" + e.getMessage();
	}
	context.put("cont_error", cont_error);

	// Get the name of the associated MBean
	ObjectName mbean = null;
	try {
	    mbean = JonasObjectName.ejbmdb(fileName, ejbName);
	} catch (Exception e) {
	    context.put("alertException", alert("Cannot get mbean: "+e.getMessage()));
	    return template;
	}
	
	// Get specific information about the MDB bean
	String info_error = "";
	String mdbName_error = "";
	try {
	    // Get the EJB's class name
	    String ejbClass = (String)JonasManagementRepr.getAttribute(mbean, "EJBClass");
	    context.put("ejbClass", ejbClass);

	    // Get the associated Jms Destination 
	    String destName = (String)JonasManagementRepr.getAttribute(mbean, "MdbJMSAssociateDestinationName");
	    context.put("destName", destName);
	} catch (Exception e) {
	    info_error = e.getMessage();
	}
	context.put("info_error", info_error);
	context.put("mdbName_error", mdbName_error);

	// Get information about resources used by the bean
	getDatasourceInfo(context, mbean);
	getConnFactoryInfo(context, mbean);
	getDestinationInfo(context, mbean);
	getMailFactoryInfo(context, mbean);

	// Return template
	return template;
    }
}
