/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Eric Hardesty
 * 
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.resource;

import java.io.IOException;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.InputStreamReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Stack;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.objectweb.jonas_rar.deployment.api.RarDeploymentDesc;
import org.objectweb.jonas_rar.deployment.api.RarDeploymentDescException;
import org.objectweb.jonas_rar.deployment.xml.ConfigProperty;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigProperty;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigPropertyImpl;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigPropertyName;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigPropertyNameImpl;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigPropertyValue;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigPropertyValueImpl;
import org.objectweb.jonas_rar.deployment.xml.PoolInit;
import org.objectweb.jonas_rar.deployment.xml.PoolInitImpl;
import org.objectweb.jonas_rar.deployment.xml.PoolMax;
import org.objectweb.jonas_rar.deployment.xml.PoolMaxImpl;
import org.objectweb.jonas_rar.deployment.xml.PoolParams;
import org.objectweb.jonas_rar.deployment.xml.PoolParamsImpl;
import org.objectweb.jonas_rar.deployment.xml.JonasResource;
import org.objectweb.jonas_rar.deployment.xml.JonasResourceImpl;
import org.objectweb.jonas_rar.deployment.xml.Jndiname;
import org.objectweb.jonas_rar.deployment.xml.JndinameImpl;
import org.objectweb.jonas_rar.deployment.xml.Rarlink;
import org.objectweb.jonas_rar.deployment.xml.RarlinkImpl;

/**
 * This class may be used to parse a resource adapter deployment descriptor
 * and generate the corresponding properties files for the jca service
 * 
 * java org.objectweb.jonas.resource.RAConfig -j jndiname -r rarlink -verbose <input_rar>
 * java org.objectweb.jonas.resource.RAConfig -u jonas-ra.xml <input_rar>
 *
 */
public class RAConfig {
    
    RarDeploymentDesc rarDD = null;
    
    static String outputFile = "META-INF/jonas-ra.xml";

    // Constructor
    public RAConfig (String fileName) {
        ClassLoader curLoader = Thread.currentThread().getContextClassLoader();
        
        try {
            rarDD = RarDeploymentDesc.getInstance(fileName, curLoader, true, false);
        } catch (Exception e) {
            System.err.println("RAConfig Exception caught "+e);
        }

    }

    public RarDeploymentDesc getRarDeploymentDesc(){
        return rarDD;
    }

    private static void updateRAR(String filename, String updatename)
    {

        ZipOutputStream zipOutput = null;
        ZipFile zipFile = null;
        Enumeration zippedFiles = null;
        ZipEntry currEntry = null;
        ZipEntry entry = null;

        try {
              File file = new File(filename);
              if (file.exists()) {
                  zipFile = new ZipFile( file.getAbsolutePath() ); 
                  //get an enumeration of all existing entries 
                  zippedFiles = zipFile.entries(); 
                  //create your output zip file
                  zipOutput = new ZipOutputStream( new FileOutputStream ( new File( file.getAbsolutePath() + "NEW") ) );
                  //Get all the data out of the previously zipped files and write it to a new ZipEntry to go into a new file archive 
                  while (zippedFiles.hasMoreElements()) {
                      //Retrieve entry of existing files
                      currEntry = (ZipEntry)zippedFiles.nextElement();
                      if (currEntry.getName().equalsIgnoreCase(outputFile)) {
                          continue; 
                      }
                      //Read data from existing file
                      zipOutput.putNextEntry(new ZipEntry(currEntry.getName()) ) ;
                      BufferedReader reader = new BufferedReader( new InputStreamReader( zipFile.getInputStream( currEntry ) ) ); 
                      int c;
                      while ((c = reader.read()) != -1) {
                          zipOutput.write(c);
                      }
                      //Commit the data
                      zipOutput.flush();
                      zipOutput.closeEntry(); 
                  }
                  //Close the old zip file
                  zipFile.close();
                  //Write the 'new' file to the archive, commit, and close 
                  entry = new ZipEntry( outputFile );
                  zipOutput.putNextEntry( entry );
                  //Read data from existing file
                  BufferedReader reader = new BufferedReader( new FileReader(updatename) );
                  int c;
                  while ((c = reader.read()) != -1) {
                      zipOutput.write(c);  
                  }
                  //Commit the data
                  zipOutput.flush();
                  zipOutput.closeEntry(); 
                  zipOutput.finish(); 
                  zipOutput.close();
                  //delete the old file and rename the new one
                  File toBeDeleted = new File ( file.getAbsolutePath() );
                  toBeDeleted.delete(); 
                  File toBeRenamed = new File ( file.getAbsolutePath() + "NEW"); 
                  toBeRenamed.renameTo( file );
              } else {
                  //create your output zip file
                  zipOutput = new ZipOutputStream( new FileOutputStream ( new File( filename ) ) );
                  //Write the file to the archive, commit, and close 
                  entry = new ZipEntry( outputFile );
                  zipOutput.putNextEntry( entry );
                  //Read data from existing file
                  BufferedReader reader = new BufferedReader( new FileReader(updatename) ); 
                  int c;
                  while ((c = reader.read()) != -1) {
                      zipOutput.write(c);  
                  }
                  //Commit the data
                  zipOutput.flush();
                  zipOutput.closeEntry(); 
                  zipOutput.finish(); 
                  zipOutput.close();
              }
        } catch (Exception ex) {
              System.out.println("Exception: "+ex);
        } 
    }        

    private static String indent(int count) {
        String front = "";
        String indentChars = "    ";
        for (int i = 0; i < count; i++) {
            front += indentChars;
        }
        return front;
    }

    public static void usage() {
        System.out.println("");
        System.out.println("Usage: java org.objectweb.jonas.resource.RAConfig <Options> <Input_File>");
        System.out.println("         to generate the jonas-ra.xml file for a Resource Adapter.");
        System.out.println("");
        System.out.println("           With Options:");
        System.out.println("             -? or -help to print this help message");
        System.out.println("             -j <jndiname>  specifies the jndiname RA");
        System.out.println("             -r <rarlink>  specifies the rar file to link to");
        System.out.println("             -u <input name> update the input rar with the specified file");   
        System.out.println("             -verbose   output the contents of the  deployment descriptor ");   
        System.out.println("");
        System.out.println("           Input_File     Resource Adapter RAR file");
        System.out.println("");
    }

    // Main Method
    public static void main (String[] args) {
        PrintStream out = null;
        boolean  isHelp          = false;
        boolean  isVerbose       = false;
        boolean  doUpdate        = false;
        String fileInputName = null;
        String jndiname = "";
        String rarlink = "";
        String updatename = "";
        
        // Get command args

        if (args.length <1) {
            System.err.println("RAConfig: Arguments missing");
            usage();
            System.exit(2);
        }
        for (int argn = 0; argn < args.length; argn++) {
            String arg = args[argn];
            if (arg.equals("-help") || arg.equals("-?")) {
                isHelp = true;
                continue;
            }
            if (arg.equals("-verbose") || arg.equals("-v")) {
                isVerbose = true;
                continue;
            }
            if (arg.equals("-update") || arg.equals("-u")) {
                doUpdate = true;
                if (argn+1 == args.length || args[++argn].startsWith("-"))
                {
                  System.err.println("*** Error with -u input");
                  usage();
                  System.exit(2);
                }  
                updatename  = args[argn];
                continue;
            }
            if (arg.equals("-jndiname") || arg.equals("-j")) {
                if (argn+1 == args.length || args[++argn].startsWith("-"))
                {
                  System.err.println("*** Error with -j input");
                  usage();
                  System.exit(2);
                }  
                jndiname  = args[argn];
                continue;
            }
            if (arg.equals("-rarlink") || arg.equals("-r")) {
                if (argn+1 == args.length || args[++argn].startsWith("-"))
                {
                  System.err.println("*** Error with -r input");
                  usage();
                  System.exit(2);
                }  
                rarlink  = args[argn];
                continue;
            }
            if (fileInputName == null) {
                fileInputName = args[argn];
                if (!fileInputName.endsWith(".rar")) {
                    System.err.println("*** Error input file must be an rar file");
                    usage();
                    System.exit(2);
                }
            }
            else {
                System.err.println("*** Error multiple input files specified");
                usage();
                System.exit(2);
            }  
        }
        
        // Usage ?
        if (isHelp) {
            usage();
            System.exit(0);
        }
        if (fileInputName == null) {
            System.err.println("*** Error missing input file");
            usage();
            System.exit(2);
        }

        if (doUpdate){
            updateRAR(fileInputName, updatename);
        } else {        
            RAConfig raxml = new RAConfig (fileInputName);
            ConfigProperty[] raProps = raxml.getRarDeploymentDesc().getRaConfigPropTags();
            if (raProps == null) {
                System.err.println("*** RAR file doesn't contain ra.xml.");
                System.exit(2);
            }

            JonasResource jRes = new JonasResourceImpl();
            Jndiname jJndi = new JndinameImpl();
            Rarlink jRlink = new RarlinkImpl();
            PoolParams jPool = new PoolParamsImpl();
            PoolInit jPoolInit = new PoolInitImpl();
            PoolMax jPoolMax = new PoolMaxImpl();
            JonasConfigProperty jCfg = null;
            JonasConfigPropertyName jCfgName = null;
            JonasConfigPropertyValue jCfgValue = null;
            
            jPoolInit.setValue("");
            jPool.setPoolInit(jPoolInit);
            jPoolMax.setValue("");
            jPool.setPoolMax(jPoolMax);
            jJndi.setValue(jndiname);
            jRes.setJndiname(jJndi);
            jRlink.setValue(rarlink);
            jRes.setRarlink(jRlink);
            jRes.setPoolParams(jPool);
            
            for(int i=0; i<raProps.length;i++){
                jCfg = new JonasConfigPropertyImpl();
                jCfgName = new JonasConfigPropertyNameImpl();
                jCfgName.setValue(raProps[i].getConfigPropertyName().getValue());
                jCfg.setJonasConfigPropertyName(jCfgName);
                jCfgValue = new JonasConfigPropertyValueImpl();
                jCfgValue.setValue("");
                jCfg.setJonasConfigPropertyValue(jCfgValue);
                jRes.addJonasConfigProperty(jCfg);
            }

            try {
                jRes.marshal(new BufferedWriter(new FileWriter("jonas-ra.xml")));
            }catch(Exception e){
                System.err.println("Error writing output");
                System.exit(2);
            }
        }
    }
}
