/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
  

package org.objectweb.jonas.resource;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;

import java.lang.reflect.Method;

import java.util.Arrays;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.Set;
import java.util.Iterator;

import javax.management.MalformedObjectNameException;
import javax.management.MBeanRegistrationException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanServer;
import javax.management.ObjectName;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.Reference;
import javax.naming.StringRefAddr;

import javax.resource.Referenceable;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionFactory;

import javax.transaction.TransactionManager;

import org.objectweb.jonas.common.JProp; 
import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.jmx.JmxService;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.jtm.TransactionService;
import org.objectweb.jonas.naming.CompNamingContext;
import org.objectweb.jonas.service.AbsServiceImpl;
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.jonas.service.ServiceManager;

import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

import org.objectweb.jonas_rar.deployment.api.RarDeploymentDesc;
import org.objectweb.jonas_rar.deployment.api.RarDeploymentDescException;
import org.objectweb.jonas_rar.deployment.xml.Connector;
import org.objectweb.jonas_rar.deployment.xml.Resourceadapter;
import org.objectweb.jonas_rar.deployment.xml.ConfigProperty;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigProperty;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigPropertyImpl;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigPropertyName;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigPropertyNameImpl;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigPropertyValue;
import org.objectweb.jonas_rar.deployment.xml.JonasConfigPropertyValueImpl;
import org.objectweb.jonas_rar.deployment.xml.PoolParams;
import org.objectweb.jonas_rar.deployment.xml.PoolParamsImpl;
import org.objectweb.jonas_rar.deployment.xml.JonasResource;
import org.objectweb.jonas_rar.deployment.xml.JonasResourceImpl;
import org.objectweb.jonas_rar.deployment.xml.Jndiname;
import org.objectweb.jonas_rar.deployment.xml.JndinameImpl;
import org.objectweb.jonas_rar.deployment.xml.Rarlink;
import org.objectweb.jonas_rar.deployment.xml.RarlinkImpl;
/**
 * JCA resource service implmentation
 * @author Philippe Coq
 * Contributor(s):
 * JOnAS 2.4 Sebastien Chassande-Barrioz (sebastien.chassande@inrialpes.fr)
 * JOnAS 3.0 Eric Hardesty (Eric.Hardesty@bull.com)
 *
 */
public class ResourceServiceImpl extends AbsServiceImpl implements 
    ResourceService,
    ResourceServiceImplMBean {

    static private Logger logger = null;
    
    // Properties for inits

    // JCA resource service configuration parameters
    public static final String RESOURCE_LIST = "jonas.service.resource.resources";
    public static final String CLASS = "jonas.service.resource.class";
    public static final String AUTOLOADDIR = "jonas.service.resource.autoloaddir";
    public static final String PARSINGWITHVALIDATION = "jonas.service.resource.parsingwithvalidation";

    public final String MANAGED_CONNECTION_FACTORY_CLASS 
        = "managedconnectionfactoryclass";
    public static final String JNDI_NAME = "jndiname";
    public static final String PRINTWRITER ="printWriter";
    public static final String MONITOR_FACTORY 
        = "org.objectweb.monolog.MonitorFactory";

    private class RAInfo {
        public Object cf;
        public ManagedConnectionFactory mcf;
        public ConnectionManager cm;
        public Connector racon;
        public JonasResource jonasra;
        public ConfigProperty [] cfgRaJonas;
        public RAInfo(Object _cf, ManagedConnectionFactory _mcf,
                      ConnectionManager _cm, Connector _racon,
                      JonasResource _jonasra, ConfigProperty [] _cfgRaJonas) {
            cf = _cf;
            mcf = _mcf;
            cm = _cm;
            racon = _racon;
            jonasra = _jonasra;
            cfgRaJonas = _cfgRaJonas;
        }
    }

    /**
     * This hashtable associates a jndiName to an instance of the local
     * class RAInfo which contains a reference to the connectionFactory
     * and the ManagedConnectionFactory
     */
    static private Hashtable jndiName2RA = new Hashtable(); 
    
    /**
     * This vector is list of RAR files that will be deployed after all
     * other RAR files have been deployed.  The problem is that these RAR
     * files are associated with another RAR file that has not been processed.
     */
     private Vector delayedRAs = new Vector(); 
     private boolean processingDelayed = false;
    
    /**
     * The transaction manager in server
     */
    private TransactionManager tm = null ;
   
    /**
     * The ConnectionManager manages connections of resource adapters
     * and more exactly the association between logical connections,
     * physical connections, local transactions and global transaction(XA)
     */
    private ConnectionManagerImpl cm = null;

    /**
     * JNDI Context
     */
    private Context ictx = null;


    private Vector resourceNames = null;

    /**
     *  The name of the JONAS_ROOT directory
     */
    private static final String JONAS_ROOT = JProp.getInstallRoot();
    
    /**
     *  The name of the rars directory
     */
    private static final String RARS_DIR = JONAS_ROOT + File.separator + "rars";
    
    
    //IMPLEMENTATION OF 'AbsServiceImpl' ABSTRACT CLASS

    /**
     * - Get the global jndi context
     * - Get the list of the resource adapters. The list is reachable in the
     * Context parameter under the name RESOURCE_LIST.
     * - Get the transaction manager into the jndi
     * - Instanciate the connection manager
     */
    public void doInit(Context ctx) {
        if(logger == null)
            logger = Log.getLogger(Log.JONAS_JCA_PREFIX);
        resourceNames = new Vector();
                
        // Avoid using NamingManager here: performance is not a goal here.
        try {
            ictx = new InitialContext();
        } catch (NamingException e) {
            logger.log(BasicLevel.ERROR,"Cannot create initial context when Resource service initializing");
            throw new ServiceException("Cannot create initial context when Resource service initializing",e);
        }

        // Add the rars of the jonas.service.resource.autoloaddir property
        String dirValue = null;
        try {
            dirValue = (String) ctx.lookup(AUTOLOADDIR);
            logger.log(BasicLevel.DEBUG, "autoloaddir= "+dirValue);
        } catch (NamingException e) {
        }
        if (dirValue != null) {
            StringTokenizer st = new StringTokenizer(dirValue, ",");
            while (st.hasMoreTokens()) {
                String dirName = st.nextToken().trim();
                addRars(dirName);
            }
        }

        // Get the list of the resource adapter names

        try {
            String rs = (String)ctx.lookup(RESOURCE_LIST);
            logger.log(BasicLevel.DEBUG, "resource list= "+rs);
            if (rs != null) {
                StringTokenizer st = new StringTokenizer(rs, ",");
                while (st.hasMoreTokens())
                    resourceNames.add(st.nextToken().trim());
            }
        } catch (NamingException e) {
            logger.log(BasicLevel.ERROR,"Cannot lookup the configuration context at Resource service starting");
            throw new ServiceException("Cannot lookup the configuration context at Resource service starting", e);
        }

        // Get a reference to the Transaction service
        try {
            tm = (TransactionManager) 
                ( (TransactionService)
                  ServiceManager.getInstance().getTransactionService()
                  ).getTransactionManager();
        } catch (ServiceException e) {
            logger.log(BasicLevel.ERROR,"Cannot get the Transaction service: "+e);
            throw e;
        }

	      // Set the XML parsing mode to no validation
	      String parsingMode = "false";
	      try {
	          parsingMode = (String) ctx.lookup(PARSINGWITHVALIDATION);
	      } catch (NamingException e) {
	          // No problem if there is no value for 'parsingwithvalidation' (false by default)
	      }
	      if ("false".equalsIgnoreCase(parsingMode)) {
	          RarDeploymentDesc.setParsingWithValidation(false);
	          logger.log(BasicLevel.DEBUG, "XML parsing without validation");
	      } else {
	          logger.log(BasicLevel.DEBUG, "XML parsing with validation");
	      }

        logger.log(BasicLevel.DEBUG, "ResourceService initialized");
    }

    public void doStart() throws ServiceException {
        // creates each resource
        String resName = null;
        for (int i = 0; i < resourceNames.size(); i++) {
            resName = (String) resourceNames.elementAt(i);
            logger.log(BasicLevel.DEBUG, "resName="+resName);
            try {
                createResourceAdapter(resName);
            } catch (Exception e) {
                logger.log(BasicLevel.ERROR,"JOnAS: Cannot create resource: "+resName+" exception: "+e);
                logger.log(BasicLevel.ERROR,"Please check if "+resName+" is correct");
                e.printStackTrace();
            }
        }
        
        // process any delayed rars
        if (!delayedRAs.isEmpty()) {
          try{
            processingDelayed = true;
            Object rarList[] = null;
            rarList = delayedRAs.toArray();
            for (int i=0;i<rarList.length;i++) {
                try {
                    createResourceAdapter((String) rarList[i]);
                } catch (Exception e) {
                    logger.log(BasicLevel.ERROR,"JOnAS: Cannot create resource: "+(String) rarList[i]+" exception: "+e);
                    logger.log(BasicLevel.ERROR,"Please verify that the rarlink is correct");
                    e.printStackTrace();
                }
            }
          }
          catch (Exception e) {
            logger.log(BasicLevel.ERROR,"ResourceService: Error with delayed RAR file deployment\n"+e);
            throw new ServiceException("ResourceService: Error with delayed RAR file deployment", e);
          }
        }

        try {
            // Register ResourceService MBean 
            ((JmxService)ServiceManager.getJmxService()).getJmxServer().registerMBean(this,
JonasObjectName.resourceService());
        } catch (ServiceException se) {
            // Jmx Service not available, do nothing
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR,"ResourceService: Cannot start the Resource service:\n"+e);
            throw new ServiceException("ResourceService: Cannot start the Resource service", e);
        }

    }

    public void doStop() throws ServiceException {
        synchronized (jndiName2RA) {
            for (Enumeration e = jndiName2RA.keys(); e.hasMoreElements();) {
                String jndiName = (String) e.nextElement();
                RAInfo rai = (RAInfo) jndiName2RA.get(jndiName);
                try {
                    // Unbind from jndi
                    ictx.unbind(jndiName);
                } catch (NamingException ex) {
                    logger.log(BasicLevel.ERROR,"ResourceService: cannot unbind ResourceAdapter"
                                +jndiName+" :\n" + ex);
                    throw new ServiceException(ex.getMessage());
                }

                // The connection manger must stop to manage the RA
                //rai.cm.cleanResourceAdapter();

                    
                try {
                    // unbind MBean
                    ((JmxService)ServiceManager.getJmxService()).getJmxServer()
                        .unregisterMBean( 
                                         JonasObjectName.resourceAdapter(jndiName));
                } catch  (MBeanRegistrationException mone) {
                    //???
                } catch  (MalformedObjectNameException mone) {
                    //???
                } catch  (InstanceNotFoundException infe) {
                    //???
                } catch  (ServiceException se) {
                    // Jmx Service not available, do nothing
                }

                logger.log(BasicLevel.DEBUG, 
                            "ResourceService: unbindResourceAdapter:"+jndiName);
            }
        }

        try {
            MBeanServer mbeanServer =
((JmxService)ServiceManager.getJmxService()).getJmxServer();
            // unregister resource Service MBean 
            mbeanServer.unregisterMBean(JonasObjectName.resourceService());
            // unregister all Resource Adaptor
            Set objectNames =
mbeanServer.queryNames(JonasObjectName.allResourceAdaptors(), null) ;
            Iterator it  = objectNames.iterator();
            while (it.hasNext()) {
                mbeanServer.unregisterMBean((ObjectName)it.next());
            }
        } catch (ServiceException se) {
            // Jmx Service not available, do nothing
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR,"EJBService: Cannot stop the Resource service:\n"+e);
            throw new ServiceException("ResourceService: Cannot stop the Resource service",e);
        }
        logger.log(BasicLevel.DEBUG, "ResourceService stopped");
    }


    private ConnectionManager createConnectionManager(String trans) throws Exception {

        ConnectionManagerImpl cm = new ConnectionManagerImpl(trans);

        Context c = new CompNamingContext("");
        c.rebind(ConnectionManagerImpl.TRANSACTION_MANAGER, tm);
        c.rebind(ConnectionManagerImpl.RESOURCE_MANAGER_EVENT_LISTENER,tm);

        // Set the MonologFactory into context of initialisation
        c.rebind(ConnectionManagerImpl.LOGGER, logger);

        cm.init(c);
        return cm;
    }

    // IMPLEMENTATION OF 'ResourceService' INTERFACE //

    /*
     * Create a new resource adapter.  This Resource Adapter is configured via 
     * xml files in the resName. 
     */ 
    public void createResourceAdapter(String  resName) throws Exception {
        int poolMax = 0;
        int poolMin = 0;
        int poolInit = 0;
        String tmpPool = null;
        ConnectionManager cm = null;
        
        logger.log(BasicLevel.DEBUG,"entering for "+resName+","+cm);
        if(!resName.endsWith(".rar")){
            resName += ".rar";
        }
        
        File f = new File(resName);
        if (!f.exists()) {
            boolean found = false;
      	    String rarFileName = null;
            // In case of the name is a rar file name, check also in
            // the JONAS_ROOT/rars directory
            rarFileName = RARS_DIR + File.separator + resName;
            File fr = new File(rarFileName);
            found = fr.exists();
            if (found) {
                resName = rarFileName;
            } else {
                logger.log(BasicLevel.ERROR, "createResourceAdapter: "+resName+" not found");
                Exception e = new NamingException(resName+" not found");
                throw e;
      	    }
      	}
        
        ClassLoader curLoader = Thread.currentThread().getContextClassLoader();
        RarDeploymentDesc radesc = RarDeploymentDesc.getInstance(resName, curLoader);

        Connector conn = radesc.getConnector();
        JonasResource jonasra = radesc.getJonasResource();
        if(conn == null && jonasra == null){
            logger.log(BasicLevel.ERROR,"ResourceService.createRA: Resource ("+resName+") must be a valid RAR file.");
            throw new Exception("resource input file incorrect");
        }

        // Need to check if rarlink is specified.  If yes and has been processed, then get the correct
        // Connector object and continue, if it has not been processed, then add this rar to the
        // deferred list and it will be processed at end of the rar files.  If no rarlink, then 
        // make sure that the connector object is valid.
        String rarVal = jonasra.getRarlink().getValue();
        ConfigProperty [] mergeConfigs = null;
        if (rarVal != null && rarVal.length() > 0) {
            conn = getConnector(rarVal);
            if (conn == null) {
                if (processingDelayed) {
                    logger.log(BasicLevel.ERROR,"ResourceService.createRA: Resource ("+resName+") with rarlink ("+rarVal+") is not valid.");
                    throw new Exception("resource input file incorrect");
                }
                delayedRAs.add(resName);
                return;
            }
            else {
                // Get the config parameters from the rarlink
                radesc.setRaConfigPropTags(conn);
                mergeConfigs = getMergeConfigs(rarVal);
                // Setup the pool parameters from the rarlink
                JonasResource jr = getJonasResource(rarVal);
                if (jr != null) {
                    if (jr.getPoolParams() != null) {
                        if (jr.getPoolParams().getPoolInit() != null) {
                            tmpPool = jr.getPoolParams().getPoolInit().getValue();
                            if (tmpPool != null && tmpPool.length() > 0) poolInit = Integer.parseInt(tmpPool);
                        }
                        if (jr.getPoolParams().getPoolMin() != null) {
                            tmpPool = jr.getPoolParams().getPoolMin().getValue();
                            if (tmpPool != null && tmpPool.length() > 0) poolMin = Integer.parseInt(tmpPool);
                        }
                        if (jr.getPoolParams().getPoolMax() != null) {
                            tmpPool = jr.getPoolParams().getPoolMax().getValue();
                            if (tmpPool != null && tmpPool.length() > 0) poolMax = Integer.parseInt(tmpPool);
                        }
                    }
                }
            }
                
        }
        else if (conn == null || jonasra == null) {
            logger.log(BasicLevel.ERROR,"ResourceService.createRA: Resource ("+resName+") must be a valid RAR file. Conn: "+conn+" Jonasra: "+jonasra);
            throw new Exception("resource input file incorrect");
        }
        
        Resourceadapter ra = conn.getResourceadapter();
        if (processingDelayed && mergeConfigs != null)
            ra.setConfigPropertyList(Arrays.asList(mergeConfigs));
        
        // Process the config values
        String mcfc = ra.getManagedconnectionfactoryClass().getValue();
        if(mcfc == null){
            logger.log(BasicLevel.ERROR,"ResourceService.createRA:"+MANAGED_CONNECTION_FACTORY_CLASS+" property not found");
            throw new Exception("configuration file incorrect");
        }
        // Instanciate the ra : mcf class
        Class mcfClass = Class.forName(mcfc);
        ManagedConnectionFactory mcf = 
            (ManagedConnectionFactory) mcfClass.newInstance();

        logger.log(BasicLevel.DEBUG, "mcf="+ mcf);

        cm = createConnectionManager(conn.getResourceadapter().getTransactionSupport().getValue());
        // Obtain the CF and connect the RA to the ConnectionManager
        // A connection factory must implements javax.resource.Referenceable
        Referenceable cf = (Referenceable) 
            mcf.createConnectionFactory(cm);

        logger.log(BasicLevel.DEBUG, "cf="+ cf);


        // Associate the resource adapter to the connectionmanager
        tmpPool = null;
        try {
            if (jonasra.getPoolParams() != null) {
                if (jonasra.getPoolParams().getPoolInit() != null) {
                    tmpPool = jonasra.getPoolParams().getPoolInit().getValue();
                    if (tmpPool != null && tmpPool.length() > 0) poolInit = Integer.parseInt(tmpPool);
                }
                if (jonasra.getPoolParams().getPoolMin() != null) {
                    tmpPool = jonasra.getPoolParams().getPoolMin().getValue();
                    if (tmpPool != null && tmpPool.length() > 0) poolMin = Integer.parseInt(tmpPool);
                }
                if (jonasra.getPoolParams().getPoolMax() != null) {
                    tmpPool = jonasra.getPoolParams().getPoolMax().getValue();
                    if (tmpPool != null && tmpPool.length() > 0) poolMax = Integer.parseInt(tmpPool);
                }
            }
        }
        catch (Exception ex) {
                ex.printStackTrace();
                logger.log(BasicLevel.ERROR,"Invalid Pool parameter from jonas-ra.xml:  Ensure that numeric values are used. "+tmpPool);
                throw new Exception("incorrect pool parameter ");
        }
        

        // Get the jndi name of the ra
        String jndiName = jonasra.getJndiname().getValue();
        mcf.setLogWriter(Log.getLogWriter("org.objectweb.jonas.jca"));

        PrintWriter pw = mcf.getLogWriter();
        if (pw != null)
            pw.println("Output from MCF");
        logger.log(BasicLevel.DEBUG, "jndiName="+ jndiName);
        
        // init the ra with setter method
        
        ConfigProperty [] cfgRa = radesc.getRaConfigPropTags();
        JonasConfigProperty [] cfgJonas = radesc.getJonasConfigPropTags();
        
        ConfigProperty [] cfgRaJonas = cfgMerge(radesc.getRaConfigPropTags(),
                                           radesc.getJonasConfigPropTags());
        int curParam = 0;
        while ( curParam < cfgRaJonas.length ){
            String fieldName = cfgRaJonas[curParam].getConfigPropertyName().getValue();
            String methodName = "set"+fieldName.substring(0,1).toUpperCase() + 
                                fieldName.substring(1);
            String fieldType = cfgRaJonas[curParam].getConfigPropertyType().getValue();
            Method[] m = mcfClass.getMethods();

            int i=0;
            while (i<m.length 
                   && (! m[i].getName().equals(methodName) 
                       || m[i].getParameterTypes().length!=1 
                       || ! (m[i].getParameterTypes())[0].getName().equals(fieldType))
                   ) {
                i++;
            }
            if (i<m.length) {
                Class [] paramtype = m[i].getParameterTypes();
                Object[] param = new Object[1];
                
                String curValue = cfgRaJonas[curParam].getConfigPropertyValue().getValue();
                
                logger.log(BasicLevel.DEBUG, "Processing Field Name: "+fieldName+" Type: "+fieldType+" Value: "+curValue);

                if (paramtype[0].equals(java.lang.Integer.TYPE) ||
                    paramtype[0].equals(java.lang.Integer.class)){
                    param[0]  = new Integer(curValue);

                } else if (paramtype[0].equals(java.lang.Boolean.TYPE) ||
                           paramtype[0].equals(java.lang.Boolean.class) ){
                    param[0]  = new Boolean(curValue);

                } else if (paramtype[0].equals(java.lang.Double.TYPE) || 
                           paramtype[0].equals(java.lang.Double.class) ){
                    param[0]  = new Double(curValue);

                } else if (paramtype[0].equals(java.lang.Byte.TYPE) ||
                           paramtype[0].equals(java.lang.Byte.class) ){
                    param[0]  = new Byte(curValue);
                        
                } else if (paramtype[0].equals(java.lang.Short.TYPE) ||
                           paramtype[0].equals(java.lang.Short.class)){
                    param[0]  = new Short(curValue);
                        
                } else if (paramtype[0].equals(java.lang.Long.TYPE) ||
                           paramtype[0].equals(java.lang.Long.class) ){
                    param[0]  = new Long(curValue);
                        
                } else if (paramtype[0].equals(java.lang.Float.TYPE) ||
                           paramtype[0].equals(java.lang.Float.class) ){
                    param[0]  = new Float(curValue);
                        
                } else if (paramtype[0].equals(java.lang.String.class)){
                    param[0]  = (String) curValue;
                    logger.log(BasicLevel.DEBUG, "Calling String method with "+curValue);
                        
                } else {
                    logger.log(BasicLevel.ERROR,"Type unsupported for setter method:"
                                +methodName);
                    throw new Exception("incorrect type for setter method ");
                }
                // invocation of the corresponding setter method on 
                // ManagedConnectionFactory instance
                try {
                    m[i].invoke(mcf,param);
                } catch (Exception e) {
                    logger.log(BasicLevel.DEBUG, "method "
                                +methodName+
                                " not found on resource adapter, check the configuration file "
                                +resName+".properties");
                }
                curParam++;
            } else {
                logger.log(BasicLevel.ERROR,"Method not found: "+methodName);
                throw new Exception("incorrect method name: "+methodName);
            }
        }
        

        ((ConnectionManagerImpl) cm).setResourceAdapter(mcf, poolInit, poolMin, poolMax);

        // Register into Jndi
        try {

            Reference ref = 
                new Reference(
                              cf.getClass().getName(),
                              ConnectionFactoryJNDIHandler.class.getName(),
                              null);
            ref.add( new StringRefAddr(JNDI_NAME, jndiName) );
            cf.setReference(ref);
            ictx.rebind(jndiName, cf);
            
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR,"ResourceService: Cannot register ResourceAdapter in naming with the name " + jndiName);
            logger.log(BasicLevel.ERROR,"ResourceService: Exception caught : "+e);
            return;
        }
        
        synchronized (jndiName2RA) {
            // Save the association (jndiName, RA)
            jndiName2RA.put(jndiName, new RAInfo(cf, mcf, cm, conn, jonasra, cfgRaJonas) );
        }

        try {
            Properties p = buildProp(cfgRa, cfgJonas);
            // MBean resourcesource registering
            ((JmxService)ServiceManager.getJmxService()).getJmxServer()
                .registerMBean(
                               new JmxResourceAdapter(p, jndiName), 
                               JonasObjectName.resourceAdapter(jndiName));
        } catch (ServiceException se) {
            // Jmx Service not available, do nothing
        }       
         logger.log(BasicLevel.WARN, "Mapping ResourceAdapter "+resName+ " on "+jndiName);

    }

    /**
     * Add recursively the rars of the specified directory. 
     * If the dir has a relative path, this path is relative from where 
     * the Application Server is launched. If the dir is not found it will 
     * be searched in $JONAS_ROOT/rars/ directory. 
     * @param dirPath the path to the directory containing the rars to
     * load.
     */
    private void addRars(String dirPath) {
        boolean found = false;

        // Look the directory relative to where the JOnAS server is launch or
        // if the path is absolute.
        File dir = new File(dirPath);
        found = dir.isDirectory();

        if (!found) {
            // Look the directory relative to the $JONAS_ROOT/rars directory
            dir = new File(RARS_DIR + File.separator + dirPath);
            found = dir.isDirectory();
        }

        if (found) {
            addRarsFrom(dir);
        } else {
            String err = "Warning: Cannot load dir: '" + dirPath + "' ";
            err += "is not a directory or directory doesn't exist";
            logger.log(BasicLevel.WARN, err);
        }
    }

    /**
     * Add the rars of the specified directory.
     * @param dir the directory from which the rars are loaded.
     * @throws ResourceServiceException if the argument is not a directory
     */
    private void addRarsFrom(File dir) throws ResourceServiceException {
        try {
            if (dir.isDirectory()) {
                File[] files = dir.listFiles();
                for (int i = 0; i < files.length; i++) {
                    if (files[i].isFile()) {
                        if (files[i].getPath().toLowerCase().endsWith(".rar")) {
                            resourceNames.add(files[i].getCanonicalPath());
                        }
                    } else {
                        addRarsFrom(files[i]);
                    }
                }
            } else {
                String err = "Cannot load dir: '" + dir.getPath();
                err += "' is not a directory";
                logger.log(BasicLevel.ERROR, err);
                throw new ResourceServiceException(err);
            }
        } catch (IOException e) {
            String err = "Invalid file name '" + dir.getPath();
            logger.log(BasicLevel.ERROR, err);
            throw new ResourceServiceException(err, e);
        }
    }


    //--------------------------------------------------------------
    //           Other Method
    //--------------------------------------------------------------

    /**
     * get the ResourceAdapter  matching the raname
     * will be used in ConnectionFactoryJNDIHandler class at lookup time
     */
    public static Object getConnectionFactory(String jndiName) {
        synchronized (jndiName2RA) {
            RAInfo rai = (RAInfo) jndiName2RA.get(jndiName);
            return (rai==null ? null : rai.cf );
        }
    }

    /**
     * MBean method
     * @return Integer Total Number of Resourcesource available in JOnAS
     */ 
    public Integer getCurrentNumberOfResource() {
        return new Integer(jndiName2RA.size());
    }
    
    private Properties buildProp(ConfigProperty [] cfgRa, JonasConfigProperty [] cfgJonas)
    {
      Properties ret = new Properties();
      for (int i=0;i<cfgRa.length;i++){
        String val = cfgRa[i].getConfigPropertyValue().getValue()!=null ? cfgRa[i].getConfigPropertyValue().getValue() : "";
        ret.setProperty(cfgRa[i].getConfigPropertyName().getValue(), val);
      }
      for (int i=0;i<cfgJonas.length;i++){
        String val = cfgJonas[i].getJonasConfigPropertyValue().getValue()!=null ? cfgJonas[i].getJonasConfigPropertyValue().getValue() : "";
        ret.setProperty(cfgJonas[i].getJonasConfigPropertyName().getValue(), val);
      }

      return ret;
    }
    
    private ConfigProperty [] cfgMerge(ConfigProperty [] cfgRa, JonasConfigProperty [] cfgJonas)
    {
      ConfigProperty [] configs = cfgRa;
      
      for (int i=0;i<cfgJonas.length;i++){
          String name = cfgJonas[i].getJonasConfigPropertyName().getValue();
          String val = cfgJonas[i].getJonasConfigPropertyValue().getValue();
          if (val != null) {
              for (int j=0;j<configs.length;j++) {
                  if (name.equalsIgnoreCase(configs[j].getConfigPropertyName().getValue())) {
                      configs[j].getConfigPropertyValue().setValue(val);
                      break;
                  }
              }
          }
      }

      return configs;
    }
    /**
     * get the ResourceAdapter Connector object matching the jndiname
     */
    private Connector getConnector(String jndiName) {
        synchronized (jndiName2RA) {
            RAInfo rai = (RAInfo) jndiName2RA.get(jndiName);
            return (rai==null ? null : rai.racon );
        }
    }

    /**
     * get the JonasResource object matching the jndiname
     */
    private JonasResource getJonasResource(String jndiName) {
        synchronized (jndiName2RA) {
            RAInfo rai = (RAInfo) jndiName2RA.get(jndiName);
            return (rai==null ? null : rai.jonasra );
        }
    }

    /**
     * get the ResourceAdapter ConfigProperty [] object matching the jndiname
     */
    private ConfigProperty [] getMergeConfigs(String jndiName) {
        synchronized (jndiName2RA) {
            RAInfo rai = (RAInfo) jndiName2RA.get(jndiName);
            return (rai==null ? null : rai.cfgRaJonas );
        }
    }
}


