/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Florent BENOIT & Ludovic BERT
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.server;

import java.net.URL;
import java.net.URLClassLoader;

/**
 * This class implements a URLClassLoader, that permits to add some URLs in it.
 * @author Ludovic Bert
 * @author Florent Benoit
 */
public class JClassLoader extends URLClassLoader {

    /**
     * Constructs a new ClassLoader with the specified URLs.
     * @param urls the URLs to add at the ClassLoader creation.
     * @param parent parent ClassLoader, null if no parent.
     */
    public JClassLoader(URL[] urls, ClassLoader parent) {
        super(urls, parent);
    }    

    /**
     * Constructs a new ClassLoader with the specified URLs.
     * Uses the default delegation parent classloader.
     * @param urls the URLs to add at the ClassLoader creation.
     */
    public JClassLoader(URL[] urls) {
        super(urls);
    }    

    public void printURLs() {
	URL[] urls = super.getURLs();
	for (int i = 0; i < urls.length; i++) {
	    System.out.println("url="+urls[i]);
	}
    }
	
    public String getClassPath() {
	String ps = System.getProperty("path.separator");
	String cp = "";
	URL[] urls = super.getURLs();
	for (int i = 0; i < urls.length; i++) {
	    cp = cp + ps + urls[i].getFile();
	}
	return cp;
    }
}
