/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Sebastien Chassande
 * Contributor(s):
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container.jorm;

import org.objectweb.jonas_ejb.container.JContainer;
import org.objectweb.jonas_ejb.container.JEntityFactory;
import org.objectweb.jonas_ejb.container.JEntitySwitch;
import org.objectweb.jonas_ejb.container.TraceEjb;
import org.objectweb.jonas_ejb.deployment.api.DeploymentCmp2Desc;
import org.objectweb.jonas_ejb.deployment.api.EjbRelationshipRoleDesc;
import org.objectweb.jonas_ejb.deployment.api.EntityCmp2Desc;
import org.objectweb.jonas_ejb.deployment.api.EntityJdbcCmp2Desc;
import org.objectweb.jonas_ejb.deployment.api.EntityDesc;
import org.objectweb.jonas_ejb.lib.JormType;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.facility.naming.basidir.BasidBinder;
import org.objectweb.jorm.naming.api.PBinderDelegate;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.util.monolog.api.BasicLevel;

import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import java.util.Iterator;

/**
 * This class is an extension of the JEntityFactory class. It initializes the
 * persitant class (the bean) in the jorm mapper. This class is abstract in
 * order to be extended by the PClassMapping generated for the Bean.
 *
 * @author Sebastien Chassande-Barrioz
 */
public abstract class JormFactory extends JEntityFactory implements PClassMapping {

    protected int relNonInit;
    protected boolean mapped;
    protected PMapper mapper = null;
    protected EntityCmp2Desc ecd = null;

    public JormFactory() {
        super();
    }

    protected abstract void setMapper(String mapperName) throws PException;
    public abstract Object getConnection(Object hints) throws PException;
    public abstract void releaseConnection(Object conn) throws PException;
    public abstract String getMIMappingFactory(EntityDesc bd);

    public void init(EntityDesc ed, JContainer c, String mapperName) {
        TraceEjb.logger.log(BasicLevel.DEBUG, ed.getEjbName());

        // First, call the super in order to have the access to the datasource
        // and the beanNaming
        super.init(ed, c);

        ecd = (EntityCmp2Desc) ed;
	try {
	    setMapper(mapperName);
	} catch (PException e) {
	    throw new EJBException("JormFactory cannot create the mapper", e);
	}

        //----------------------- JORM INITIALIZATION -----------------------//
        TraceEjb.logger.log(BasicLevel.DEBUG, "Jorm initialisation");
        PBinderDelegate binder = null;
        Class binderClass = null;
        short binderCT = 0;
        try {
            // Instanciate the binder
            try {
                binderClass = Class.forName(ecd.getJormBinderClassName());
                binder = (PBinderDelegate) binderClass.newInstance();
            }
            catch (ClassNotFoundException e) {
                binderClass =
                    c.getClassLoader().loadClass(ecd.getJormBinderClassName());
                binder = (PBinderDelegate) binderClass.newInstance();
            }
            if (ecd.hasPrimaryKeyField()) {
                binderCT = JormType.getCodingType(ecd.getCmpFieldDesc(ecd.getPrimaryKeyFieldName()).getFieldType());
                ((BasidBinder) binder).setCodingType(binderCT);
            }
            else {
                //TODO:
                // - take the initial values of the primary key
                // - put them into the generated binder (use a PNG)
            }

            TraceEjb.logger.log(BasicLevel.DEBUG, "binder "
				+ ecd.getJormBinderClassName() + "instanciated");

            // Link the binder and the PClassMapping
            binder.setPClassMapping(this);
            setPBinder(binder);
            TraceEjb.logger.log(BasicLevel.DEBUG, "binder linked to the mapping");
        } catch (Exception e) {
            TraceEjb.logger.log(BasicLevel.ERROR, "Impossible to create the binder", e);
            throw new EJBException(
				   "Impossible to create the binder: bean:" + ecd.getEjbName(), e);
        }

        //for each reference assignes the PNamingcontext if it is possible.
	relNonInit = 0;
        try {
            for (Iterator it = ecd.getEjbRelationshipRoleDescIterator(); it.hasNext();) {
                EjbRelationshipRoleDesc rsr = (EjbRelationshipRoleDesc) it.next();
                JormFactory pcm2 = (JormFactory)
                    c.getBeanFactory(rsr.getLinkedEntityCmp2Desc().getEjbName());
                TraceEjb.logger.log(BasicLevel.DEBUG,
				    "treatement of the relation "+rsr.getEjbRelationDesc().getName()
				    +": current-bean=" + ecd.getEjbName()
				    +", source-bean=" + rsr.getEjbName()
				    +", dest-bean="+rsr.getLinkedEntityCmp2Desc().getEjbName()
				    +", cmr-field="+rsr.getCmrFieldName());

                PClassMapping gcm = null;
                if (rsr.hasCmrField()) {
                    //Multivalued relation
                    if (rsr.isMultiValuedCMR()) {
                        // Instanciate a GenClassMapping and link it
                        gcm = newGCMInstance(mapper.getMapperName());
                        gcm.init(mapper, null);
                        setGenClassMapping(rsr.getCmrFieldName(), gcm);
                        TraceEjb.logger.log(BasicLevel.DEBUG,
					    "assign a GenClassMapping for the CMR "
					    + rsr.getCmrFieldName() + " / gcm=" + gcm);

                        PBinderDelegate gcmBinder = null;
                        try {
                            gcmBinder = (PBinderDelegate) binderClass.newInstance();
                        } catch (Exception e) {
                            TraceEjb.logger.log(BasicLevel.ERROR,
						"Impossible to create the binder of the GCM bean: "
						+ ecd.getEjbName() + " / CMR: "
						+ rsr.getCmrFieldName(), e);
                            throw new EJBException(
						   "Impossible to create the binder of the GCM bean: "
						   + ecd.getEjbName() + " / CMR: "
						   + rsr.getCmrFieldName(), e);
                        }
                        if (ecd.hasPrimaryKeyField()) {
                            ((BasidBinder) gcmBinder).setCodingType(binderCT);
                        } else {
                            //TODO:
                            // - take the initial values of the primary key
                            // - put them into the generated binder (use a PNG)
                        }
                        gcm.setPBinder(gcmBinder);
                        gcmBinder.setPClassMapping(gcm);
                        setPNamingContext(rsr.getCmrFieldName(), gcmBinder, true);
                    }

                    // As the bean with which the current bean is in relation
                    // may be not load, the test of pcm2!=null is needed
                    if (pcm2 != null) {
                        TraceEjb.logger.log(BasicLevel.DEBUG, "Pnc Assignement");
                        if (rsr.isMultiValuedCMR())
                            gcm.setPNamingContext(pcm2.getPBinder(), true);
                        else
                            setPNamingContext(rsr.getCmrFieldName(), pcm2.getPBinder(), true);
                    } else {
                        TraceEjb.logger.log(BasicLevel.DEBUG, "the Pnc is not reachable currently");
                        // The PNamingContext has not been set.
                        relNonInit++;
                    }
                }
                EjbRelationshipRoleDesc rsr2 = rsr.getOppositeEjbRelationshipRoleDesc();
                if (pcm2 != null && rsr2.hasCmrField()) {
                    // This case appears when the scheduling of the bean adding
                    // in the container is bad (or cyclic relation): A bean has
                    // been load and have a relation to current bean. We must
                    // therfore set the naming context on the PClassMapping of
                    // the first bean for the opposite CMR field
                    TraceEjb.logger.log(BasicLevel.DEBUG,
					"later Pnc assignement of the opposite CMR field: "
					+ rsr2.getCmrFieldName());
                    pcm2.configurePnc(rsr2.getCmrFieldName(), getPBinder(), rsr2.isMultiValuedCMR());
                }
            }
        } catch (EJBException e) {
            throw e;
        } catch (Exception e) {
            TraceEjb.logger.log(BasicLevel.ERROR,
				"mpossible to assign the naming context to the PClassMapping", e);
            throw new EJBException("Impossible to assign the naming context to the PClassMapping", e);
        }
	mapped = false;
	if (relNonInit == 0)
	    mapClass();
    }

    /**
     * It assignes the PNamingContext which manager a relation. In addition, if
     * all
     */
    public void configurePnc(String n,
                             PNamingContext pnc,
                             boolean isMultiple) throws PException {
        if (isMultiple)
            getGenClassMapping(n).setPNamingContext(pnc, true);
        else
            setPNamingContext(n, pnc, true);
        relNonInit--;
        if (relNonInit == 0)
            mapClass();
    }

    private void mapClass() {
	if (mapped)
	    throw new EJBException("The class is already mapped");

	// Drop or Create tables, depending on what is in the DD
	byte dataAction;
	switch (ecd.getCleanupPolicy()) {
	case EntityDesc.CLEANUP_REMOVEDATA:
	    dataAction = PClassMapping.CLEANUP_REMOVEDATA;
	    break;
	case EntityDesc.CLEANUP_REMOVEALL:
	    dataAction = PClassMapping.CLEANUP_REMOVEALL;
	    break;
	case EntityDesc.CLEANUP_NONE:
	    dataAction = PClassMapping.CLEANUP_DONOTHING;
	    break;
	case EntityDesc.CLEANUP_CREATE:
	    dataAction = PClassMapping.CREATE_STRUCTURE_IF_NEEDED;
	    break;
	default:
	    throw new EJBException(
				   "Unknown cleanup policy: " + ecd.getCleanupPolicy());
	}

        // Map the user class into the mapper
        Object conn = null;
        try {
            conn = getConnection(null);
            mapper.map(conn, this, dataAction, false);
	    mapped = true;
            TraceEjb.logger.log(BasicLevel.DEBUG, getClassName() + " is mapped");
        } catch (PException pe) {
            Exception e = pe;
            while((e instanceof PException) && ((PException) e).getNestedException()!=null)
                e = ((PException) e).getNestedException();
            TraceEjb.logger.log(BasicLevel.ERROR, "Impossible to map the class on the rdb mapper", e);
            throw new EJBException("Impossible to map the class on the rdb mapper", e);
        }
        finally {
            try {
                releaseConnection(conn);
            }
            catch (PException e) {
                TraceEjb.logger.log(BasicLevel.ERROR, "impossible to close a connection", e);
            }
        }
    }

    public void stop() {
	super.stop();
	try {
	    mapper.unmap(getClassName());
	    mapped = false;
	} catch (PException e) {
	    TraceEjb.logger.log(BasicLevel.ERROR,
				"Impossible to unmap the class " + getClassName(), e);
	}
    }

    /**
     * This method is overrided in order to specify the JEntityContext class
     * which must be instanciated.
     * Create a new instance of the bean and its EntityContext
     * In case of CMP, the bean class is derived to manage entity persistence.
     */
    protected org.objectweb.jonas_ejb.container.JEntityContext createNewInstance() throws Exception {

        TraceEjb.interp.log(BasicLevel.DEBUG, "JormFactory.createNewInstance");

        // create the bean instance
        EntityBean bean = null;
        try {
            bean = (EntityBean) beanclass.newInstance();
        }
        catch (InstantiationException e) {
            TraceEjb.logger.log(BasicLevel.ERROR,ejbname + " Cannot instantiate bean");
            throw e;
        }
        catch (IllegalAccessException e) {
            TraceEjb.logger.log(BasicLevel.ERROR,ejbname + " Cannot instantiate bean");
            throw e;
        }
        TraceEjb.interp.log(BasicLevel.DEBUG, "JormFactory.createNewInstance: bean class instancied :" + beanclass.getName());

        // create a new EntityContext and bind it to the instance
        org.objectweb.jonas_ejb.container.JEntityContext ec = new
            org.objectweb.jonas_ejb.container.jorm.JEntityContext(this, bean);
        bean.setEntityContext(ec);

        // If too much instances, ask swapper to clean memory.
        instanceCount++;
        if (maxCacheSize != 0 && instanceCount > maxCacheSize) {
            cont.registerBF(this);
        }

        return ec;
    }

    public JEntitySwitch getJEntitySwitch() {
        Object result = null;
	String cn = ((EntityCmp2Desc) dd).getJormBindingClassName(mapper.getMapperName());
        try {
	    result = getContainer().getClassLoader().loadClass(cn).newInstance();
        } catch (ClassNotFoundException e1) {
	    TraceEjb.logger.log(BasicLevel.ERROR,"Impossible to create a new JEntitySwitch as specified in BeanNaming: "+ cn, e1);
	    return null;
        } catch (Exception e) {
            TraceEjb.logger.log(BasicLevel.ERROR,"Impossible to create a new JEntitySwitch as specifief in BeanNaming: "+ cn, e);
            return super.getJEntitySwitch();
        }
        try {
            ((PBinding) result).init(this);
        } catch (PException e) {
            TraceEjb.logger.log(BasicLevel.ERROR,"Impossible to initialized the new JEntitySwitch as specifief in BeanNaming: "+ cn, e);
        }
        return (JEntitySwitch) result;
    }

    protected PClassMapping newGCMInstance(String mapperName) throws Exception {
	int idx = mapperName.indexOf(".");
	String mn;
	if (idx != -1)
	    mn = mapperName.substring(0, idx);
	else
	    mn = mapperName;
	return (PClassMapping) Class.forName("org.objectweb.jorm.interp.genclass." + mn + ".lib."
					     + Character.toUpperCase(mn.charAt(0))
					     + mn.substring(1, mn.length()) + "GenClassMapping"
					     ).newInstance();
    }
}
