/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Christophe Ney
 * Contributor(s): France Telecom R&D
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.deployment.api;

import org.objectweb.jonas_ejb.deployment.xml.*;
import org.objectweb.jonas_ejb.lib.MappingBuilder;
import org.objectweb.jonas_ejb.lib.RdbMappingBuilder;


/**
 * Class to hold meta-information related to an CMP v2 entity bean with jdbc data store.
 * @author Christophe Ney [cney@batisseurs.com]
 */
public class EntityJdbcCmp2Desc extends EntityCmp2Desc {

    protected String jdbcTableName;
    protected String dsname;

    /**
     * constructor
     */
    public EntityJdbcCmp2Desc(ClassLoader classLoader,
                              Entity ent,
                              AssemblyDescriptor asd,
                              JonasEntity jEnt,
                              DeploymentCmp2Desc dc2d)
            throws DeploymentDescException {

        super(classLoader, ent, asd, jEnt, dc2d);
	
        // check for jdbcMapping
        JdbcMapping jm = jEnt.getJdbcMapping();
        if (jm == null) {
            throw new DeploymentDescException("jdbc-mapping missing for bean "+ent.getEjbName());
	}

        // jndi name of the datasource
        dsname = jm.getJndiName().getValue();

    }

    /**
     * Get the datasource jndi name
     * @return String representation of the jndi name
     */
    public String getDatasourceJndiName() {
        return dsname;
    }

    /**
     * return the mapping builder to use for the entity bean
     */
    public MappingBuilder getMappingBuilder() {
        return new RdbMappingBuilder();
    }

    /**
     * String representation of the object for test purpose
     * @return String representation of this object
     */
    public String toString() {
        StringBuffer ret = new StringBuffer();
        ret.append(super.toString());
        ret.append("\ngetDatasourceJndiName()="+getDatasourceJndiName());
        return ret.toString();
    }


}
