/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import junit.framework.Test;
import junit.framework.TestSuite;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This is a test suite about the different cases of the types of the entity bean's fields.
 * It run suites for CMP version 1, and for CMP version 2.
 * Beans used: etype/*
 * @author Helene Joanin
 */

public class C_Etype extends JTestCase {

    public C_Etype(String name) {
	super(name);
    }

    public static Test suite() {

	TestSuite suite = new TestSuite();

	// primitive double
	suite.addTest(F_EtypePdoubleEC.suite());
	suite.addTest(F_EtypePdoubleEC2.suite());

	// java.lang.Double
	suite.addTest(F_EtypeOdoubleEC.suite());
	suite.addTest(F_EtypeOdoubleEC2.suite());

	// primitive float
	suite.addTest(F_EtypePfloatEC.suite());
	suite.addTest(F_EtypePfloatEC2.suite());

	// java.lang.Float
	suite.addTest(F_EtypeOfloatEC.suite());
	suite.addTest(F_EtypeOfloatEC2.suite());

	// primitive long
	suite.addTest(F_EtypePlongEC.suite());
	suite.addTest(F_EtypePlongEC2.suite());

	// java.lang.Long
	suite.addTest(F_EtypeOlongEC.suite());
	suite.addTest(F_EtypeOlongEC2.suite());

	// primitive short
	suite.addTest(F_EtypePshortEC.suite());
	suite.addTest(F_EtypePshortEC2.suite());

	// java.lang.Short
	suite.addTest(F_EtypeOshortEC.suite());
	suite.addTest(F_EtypeOshortEC2.suite());

	// java.util.Date (CMP version 2 only)
	suite.addTest(F_EtypeOutildateEC2.suite());

	// java.sql.Date (CMP version 1 only)
	suite.addTest(F_EtypeOsqldateEC.suite());

	// primitive boolean
	suite.addTest(F_EtypePbooleanEC.suite());
	suite.addTest(F_EtypePbooleanEC2.suite());

	// java.lang.Boolean
	suite.addTest(F_EtypeObooleanEC.suite());
	suite.addTest(F_EtypeObooleanEC2.suite());

	// serializable type
	suite.addTest(F_EtypeOserEC.suite());
	suite.addTest(F_EtypeOserEC2.suite());

	return suite;

    }

    public static void main (String args[]) {
	junit.textui.TestRunner.run(suite());
    }
}
