/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.rmi.RemoteException;
import java.util.Enumeration;
import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.objectweb.jonas.jtests.beans.relation.oob.ARemote;
import org.objectweb.jonas.jtests.beans.relation.oob.AHomeRemote;
import org.objectweb.jonas.jtests.beans.relation.oob.BRemote;
import org.objectweb.jonas.jtests.beans.relation.oob.BHomeRemote;
import org.objectweb.jonas.jtests.util.JTestCase;
import javax.ejb.ObjectNotFoundException;
/**
 * This is an advanced test suite for home interface on entity bean CMP2.
 * Beans used: oob/A1, oob/A2
 *
 */
public class F_Relation_oobEC2 extends A_Relation {


    private static String BEAN_HOME_A = "relation_oob_AHome";
    private static String BEAN_HOME_B = "relation_oob_BHome";
    protected static AHomeRemote ahome = null;
    protected static BHomeRemote bhome = null;

   
    public F_Relation_oobEC2(String name) {
	super(name);
    }

    protected void setUp() {
    	super.setUp();
	boolean ok = false;
	int nbtry = 0;
	while (!ok && nbtry < 3) {
	    if (!isInit) {
		// load bean if not loaded yet
		useBeans("oob", false);
		// lookup home used in the tests
		try {
		    ahome = (AHomeRemote) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME_A),
								      AHomeRemote.class);
		    bhome = (BHomeRemote) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME_B),
								      BHomeRemote.class);
		} catch (NamingException e) {
		    fail("Cannot get bean home: " + e.getMessage());
		}
		// check if tables have been initialized
		try {
		    ahome.findByPrimaryKey("a2");
		} catch (Exception e) {
		    // Make the initialization needed for the tests
		    try {
			utx.begin();
			ARemote a1 = ahome.create("a1");
			ARemote a2 = ahome.create("a2");
			ahome.create("a3");
			bhome.create("b1");
			bhome.create("b2");
			bhome.create("b3");
			a1.assignB("b1");
			a2.assignB("b2");
		    } catch (Exception i) {
			fail("InitialState creation problem: "+i);
		    } finally {
			try {
			    utx.commit();
			} catch (Exception ii) {
			}
		    }
		}
		isInit = true;
	    }
	    // Check that all is OK. Sometimes, a test has failed and has corrupted
	    // the bean state in the database. We must unload and reload the bean then.
	    nbtry++;
	    try {
		if (initStateOK()) {
		    ok = true;
		}
	    } catch (Exception e) {
	    }
	    if (!ok) {
		isInit = false;
		unloadBeans("oob");
	    }
	}
    }

    /*
     * Check that we are in the same state as after the tables creation for thoses beans A and B
     * (ie if it is the initial state)
     */
    boolean initStateOK() throws Exception {
	boolean isOk = true;

	msgerror = new StringBuffer();

        ARemote a1 = ahome.findByPrimaryKey("a1");
	BRemote b1 = bhome.findByPrimaryKey("b1");
        ARemote a2 = ahome.findByPrimaryKey("a2");
	BRemote b2 = bhome.findByPrimaryKey("b2");
        ARemote a3 = ahome.findByPrimaryKey("a3");
	BRemote b3 = bhome.findByPrimaryKey("b3");
	String idB1 = a1.retrieveB();
        String idB2 = a2.retrieveB();
	String idB3 = a3.retrieveB();
	String idA1 = b1.retrieveA();
        String idA2 = b2.retrieveA();
	String idA3 = b3.retrieveA();

        if (idB1 != null && !idB1.equals("b1")){
            isOk = false;
            msgerror.append("\nWrong relation for  a1->b1"
		       +"(expected: A1.retrieveB()='b1'"
		       + ", found:" + idB1 + ")");
        }
	if (idB2!=null && !idB2.equals("b2")){
            isOk = false;
            msgerror.append("\nWrong relation for  a2->b2"
		       +"(expected: A2.retrieveB()='b2'"
		       + ", found:" + idB2 + ")");
	}
	if (idA1!=null && !idA1.equals("a1")){
            isOk = false;
            msgerror.append("\nWrong relation for  b1->a1"
		       +"(expected: B1.retrieveA()='a1'"
		       + ", found:" + idA1 + ")");
	    System.out.println("Wrong relation for b1->a1");
	}
	if (idA2!=null && !idA2.equals("a2")){
            isOk = false;
            msgerror.append("\nWrong relation for  b2->a2"
		       +"(expected: B2.retrieveA()='a2'"
		       + ", found:" + idA2 + ")");
	}
	if (idA3!=null){
            isOk = false;
            msgerror.append("\nWrong relation for  a3->b3"
		       +"(expected: A3.retrieveB()=null"
		       + ", found:" + idA3 + ")");
	}
	if (idB3!=null){
            isOk = false;
            msgerror.append("\nWrong relation for  b3->a3"
		       +"(expected: B3.retrieveA()=null"
		       + ", found:" + idB3 + ")");
	}
	return isOk;
    }


    /**
     * Check that the bean 'a3' has no relation.
     */
    public void _testBasicGetEmpty(int tx) throws Exception {
	String  idB = null;
	if ((tx == TX_CALL) || (tx == TX_RB)) {
	    utx.begin();
	}
	ARemote a = ahome.findByPrimaryKey("a3");
	if (tx == TX_CONT) {
	    idB = a.retrieveBInNewTx();
	} else {
	    idB = a.retrieveB();
	}
	if (tx == TX_CALL) {
	    utx.commit();
	} else if (tx == TX_RB) {
	    utx.rollback();
	}
        checkIsInitialState();
    }

    public void testBasicGetEmptyTxNo() throws Exception {
	_testBasicGetEmpty(TX_NO);
    }
    public void testBasicGetEmptyTxCall() throws Exception {
	_testBasicGetEmpty(TX_CALL);
    }
    public void testBasicGetEmptyTxCont() throws Exception {
	_testBasicGetEmpty(TX_CONT);
    }

    /**
     * Set a relation to empty : a3.assignB("b3")
     */
    public void _testBasicSetEmpty(int tx) throws Exception {
	if ((tx == TX_CALL) || (tx == TX_RB)) {
	    utx.begin();
	}
	ARemote a = ahome.findByPrimaryKey("a3");
	if (tx == TX_CONT) {
	    a.assignBInNewTx("b3");
	} else {
	    a.assignB("b3");
	}
	if (tx == TX_CALL) {
	    utx.commit();
	} else if (tx == TX_RB) {
	    utx.rollback();
	}
	// checking
	String idB = a.retrieveB();
	if (tx != TX_RB) {
	    assertEquals("Wrong assign null for relation a3->b3 : ","b3", idB);
	} else {
	    assertNull("Wrong assign null for relation a3->b3 : ", idB);
	}
	// undo
	if (tx != TX_RB) {
	    a.assignB(null);
	}
        checkIsInitialState();
    }

    public void testBasicSetEmptyTxNo() throws Exception {
	_testBasicSetEmpty(TX_NO);
    }
    public void testBasicSetEmptyTxCall() throws Exception {
	_testBasicSetEmpty(TX_CALL);
    }
    public void testBasicSetEmptyTxCont() throws Exception {
	_testBasicSetEmpty(TX_CONT);
    }
    public void testBasicSetEmptyTxRb() throws Exception {
	_testBasicSetEmpty(TX_RB);
    }

    /**
     * Set a relation to empty : a3.assignB(null)
     */
    public void _testBasicSetEmptyNull(int tx) throws Exception {
	if ((tx == TX_CALL) || (tx == TX_RB)) {
	    utx.begin();
	}
	ARemote a = ahome.findByPrimaryKey("a1");
	if (tx == TX_CONT) {
	    a.assignBInNewTx(null);
	} else {
	    a.assignB(null);
	}
	if (tx == TX_CALL) {
	    utx.commit();
	} else if (tx == TX_RB) {
	    utx.rollback();
	}
	// checking
	String idB = a.retrieveB();
	if (tx == TX_RB) {
	    assertEquals("Wrong assign null for relation a1->b1: ","b1", idB);
	} else {
	    assertNull("Wrong assign null for relation a1->b1: " + idB, idB);
	}
	// undo
	if (tx != TX_RB) {
	    a.assignB("b1");
	}
        checkIsInitialState();
    }

    public void testBasicSetEmptyNullTxNo() throws Exception {
	_testBasicSetEmptyNull(TX_NO);
    }
    public void testBasicSetEmptyNullTxCall() throws Exception {
	_testBasicSetEmptyNull(TX_CALL);
    }
    public void testBasicSetEmptyNullTxCont() throws Exception {
	_testBasicSetEmptyNull(TX_CONT);
    }
    public void testBasicSetEmptyNullTxRb() throws Exception {
	_testBasicSetEmptyNull(TX_RB);
    }



    /**
     * test coherence relation one to one bidirectionnel,
     * A1.assignB(B2) => A1.retreiveB()=B2 && B2.retreiveA()=A1 && B1.retreiveA()=null && A2.retreiveB()=null
     */
    public void _testCohSetOne(int tx) throws Exception {

	if ((tx == TX_CALL) || (tx == TX_RB)) {
	    utx.begin();
	}
	ARemote a1 = ahome.findByPrimaryKey("a1");
	BRemote b1 = bhome.findByPrimaryKey("b1");
	ARemote a2 = ahome.findByPrimaryKey("a2");
	BRemote b2 = bhome.findByPrimaryKey("b2");
	ARemote a3 = ahome.findByPrimaryKey("a3");
	BRemote b3 = bhome.findByPrimaryKey("b3");
	// change the relation
	if (tx == TX_CONT) {
	    a1.assignBInNewTx("b2");
	} else {
	    a1.assignB("b2");
	}
	if (tx == TX_CALL) {
	    utx.commit();
	} else if (tx == TX_RB) {
	    utx.rollback();
	}
	if (tx != TX_RB) {
	    // Verify expected result
	    assertNull("Bad coherence of relation null : expected for a2.retreiveB() found :"+a2.retrieveB(),a2.retrieveB());
	    assertNull("Bad coherence of relation null : expected for b1.retreiveA() found :"+b1.retrieveA(),b1.retrieveA());
	    assertEquals("Bad coherence of relation a1 : expected for b2.retreiveA() found :"+b2.retrieveA(),"a1", b2.retrieveA());
	    assertEquals("Bad coherence of relation b2 : expected for a1.retreiveB() found :"+a1.retrieveB(),"b2", a1.retrieveB());

	    // undo
	    a1.assignB("b1");
	    a2.assignB("b2");
	}
	checkIsInitialState();
    }

    public void testCohSetOneTxNo() throws Exception {
	_testCohSetOne(TX_NO);
    }
    public void testCohSetOneTxCall() throws Exception {
	_testCohSetOne(TX_CALL);
    }

    /**
     * test coherence relation one to one bidirectionnel,
     * A3.assignB(B3)=>A3.retreiveB()==B3 && B3.retreiveB()==A3
     */
    public void _testCohWithoutRelation(int tx) throws Exception {

	if ((tx == TX_CALL) || (tx == TX_RB)) {
	    utx.begin();
	}
	ARemote a3 = ahome.findByPrimaryKey("a3");
	BRemote b3 = bhome.findByPrimaryKey("b3");
	// change the relation
	if (tx == TX_CONT) {
	    a3.assignBInNewTx("b3");
	} else {
	    a3.assignB("b3");
	}
	if (tx == TX_CALL) {
	    utx.commit();
	} else if (tx == TX_RB) {
	    utx.rollback();
	}
	if (tx != TX_RB) {
	    // Verify expected result
	    assertEquals("Bad coherence of relation : b3 expected for a3.retreiveB() found :"+a3.retrieveB(),"b3", a3.retrieveB());
	    assertEquals("Bad coherence of relation : a3 expected for b3.retreiveA() found :"+b3.retrieveA(),"a3", b3.retrieveA());

	    // undo
	    a3.assignB(null);
	}
	checkIsInitialState();
    }

    public void testBasicCohWithoutRelation() throws Exception {
	_testCohWithoutRelation(TX_NO);
    }
    public void testBasicCohWithoutRelationTxCall() throws Exception {
	_testCohWithoutRelation(TX_CALL);
    }

    /**
     * test coherence relation one to one bidirectionnel,
     * A1.assignB(B3)=>A1.retreiveB()==B3 && B1.retreiveB()==null &&  B2.retreiveB()=null
     */
    public void _testCohAlreadyAssign(int tx) throws Exception {

	if ((tx == TX_CALL) || (tx == TX_RB)) {
	    utx.begin();
	}
	ARemote a1 = ahome.findByPrimaryKey("a1");
	BRemote b1 = bhome.findByPrimaryKey("b1");
	BRemote b3 = bhome.findByPrimaryKey("b3");
	// change the relation
	if (tx == TX_CONT) {
	    a1.assignBInNewTx("b3");
	} else {
	    a1.assignB("b3");
	}
	if (tx == TX_CALL) {
	    utx.commit();
	} else if (tx == TX_RB) {
	    utx.rollback();
	}
	if (tx != TX_RB) {
	    // Verify expected result
	    assertEquals("Bad coherence of relation : b3 expected for a1.retreiveB() found :"+a1.retrieveB(),"b3", a1.retrieveB());
	    assertNull("Bad coherence of relation : null expected for b1.retreiveA() found :"+b1.retrieveA(), b1.retrieveA());
	    assertEquals("Bad coherence of relation : a1 expected for b2.retreiveA() found :"+b3.retrieveA(), "a1", b3.retrieveA());

	    // undo
	    a1.assignB("b1");
	}
	checkIsInitialState();
    }

    public void testCohAlreadyAssign() throws Exception {
	_testCohAlreadyAssign(TX_NO);
    }
    public void testCohAlreadyAssignTxCall() throws Exception {
	_testCohAlreadyAssign(TX_CALL);
    }

    /**
     * test coherence relation one to one bidirectionnel,
     * A1.assignB(null)=>A1.retreiveB()==null && B1.retreiveB()==null
     */
    public void _testCohSetNull(int tx) throws Exception {

	if ((tx == TX_CALL) || (tx == TX_RB)) {
	    utx.begin();
	}
	ARemote a1 = ahome.findByPrimaryKey("a1");
	BRemote b1 = bhome.findByPrimaryKey("b1");
	ARemote a2 = ahome.findByPrimaryKey("a2");
	BRemote b2 = bhome.findByPrimaryKey("b2");
	ARemote a3 = ahome.findByPrimaryKey("a3");
	BRemote b3 = bhome.findByPrimaryKey("b3");
	// change the relation
	if (tx == TX_CONT) {
	    a1.assignBInNewTx(null);
	} else {
	    a1.assignB(null);
	}
	if (tx == TX_CALL) {
	    utx.commit();
	} else if (tx == TX_RB) {
	    utx.rollback();
	}
	if (tx != TX_RB) {
	    // Verify expected result
	    assertNull("Bad coherence of relation : null expected for a1.retreiveB() found :"+a1.retrieveB(), a1.retrieveB());
	    assertNull("Bad coherence of relation : null expected for b1.retreiveA() found :"+b1.retrieveA(), b1.retrieveA());

	    // undo
	    a1.assignB("b1");
	}
	checkIsInitialState();
    }

    public void testCohSetNull() throws Exception {
	_testCohSetNull(TX_NO);
    }
    public void testCohSetNullTxCall() throws Exception {
	_testCohSetNull(TX_CALL);
    }

    /**
     * test coherence relation one to one bidirectionnel, 
     * A1.remove=>A1removed && B1.retreiveA()==null 
     */
    public void _testCohRemoveA(int tx) throws Exception {
	if (tx == TX_CONT) {
	    // The transaction attribute of the remove method is TX_SUPPORT,
	    // so the transaction cannot be initiate by the container
	    fail("Transaction cannot be initiate by the container for this test");
	}
	if ((tx == TX_CALL) || (tx == TX_RB)) {
	    utx.begin();
	}
	BRemote b1 = bhome.findByPrimaryKey("b1");
	// remove the bean
	ahome.remove("a1");
	if (tx == TX_CALL) {
	    utx.commit();
	} else if (tx == TX_RB) {
	    utx.rollback();
	}
	if (tx != TX_RB) {
	    // Verify expected result
	    assertNull("Bad coherence of relation : null expected for b1.retreiveB() found :"+b1.retrieveA(), b1.retrieveA());	
	    boolean not_found=false;
	    try {
		ARemote a1 = ahome.findByPrimaryKey("a1");
	    } catch (ObjectNotFoundException e) {
		not_found=true;
	    }
	    assertTrue("a1 is not removed",not_found);
	    // undo
	    ahome.create("a1");
	    ARemote a1 = ahome.findByPrimaryKey("a1");
	    a1.assignB("b1"); 
	}     
	checkIsInitialState();
    }

    public void testCohRemoveATxNo() throws Exception {
	_testCohRemoveA(TX_NO);
    }
    public void testCohRemoveATxCall() throws Exception {
	_testCohRemoveA(TX_CALL);
    }
    public void testCohRemoveATxRb() throws Exception {
	_testCohRemoveA(TX_RB);
    }

    /**
     * test coherence relation one to one bidirectionnel, 
     * Ahome.remove(a1)=>A1removed && B1.retreiveA()==null
     * Same as _testCohRemoveA except that the called remove method is on the bean
     * instead of the home.
     */
    public void _testCohBeanRemoveA(int tx) throws Exception {
	if (tx == TX_CONT) {
	    // The transaction attribute of the remove method is TX_SUPPORT,
	    // so the transaction cannot be initiate by the container
	    fail("Transaction cannot be initiate by the container for this test");
	}
   	       
	if ((tx == TX_CALL) || (tx == TX_RB)) {
	    utx.begin();
	}
	ARemote a1 = ahome.findByPrimaryKey("a1");
	BRemote b1 = bhome.findByPrimaryKey("b1");
	a1.remove();
	if (tx == TX_CALL) {
	    utx.commit();
	} else if (tx == TX_RB) {
	    utx.rollback();
	}
	if (tx != TX_RB) {
	    // Verify expected result
	    assertNull("Bad coherence of relation : null expected for b1.retreiveB() found :"+b1.retrieveA(), b1.retrieveA());	
	    boolean not_found=false;
	    try {
		a1 = ahome.findByPrimaryKey("a1");
	    } catch (ObjectNotFoundException e) {
		not_found=true;
	    }
	    assertTrue("a1 is not removed",not_found);
	    // undo
	    ahome.create("a1");
	    a1 = ahome.findByPrimaryKey("a1");
	    a1.assignB("b1"); 
	}     
	checkIsInitialState();
    }

    public void testCohBeanRemoveATxNo() throws Exception {
	_testCohBeanRemoveA(TX_NO);
    }
    public void testCohBeanRemoveATxCall() throws Exception {
	_testCohBeanRemoveA(TX_CALL);
    }
    public void testCohBeanRemoveATxRb() throws Exception {
	_testCohBeanRemoveA(TX_RB);
    }

    /**
     * test coherence relation one to one bidirectionnel, 
     * Bhome.remove(b1)=>B1 removed && A1.retreiveB()==null 
     */
    public void _testCohRemoveB(int tx) throws Exception {

	if (tx == TX_CONT) {
	    // The transaction attribute of the remove method is TX_SUPPORT,
	    // so the transaction cannot be initiate by the container
	    fail("Transaction cannot be initiate by the container for this test");
	}
   	       
	if ((tx == TX_CALL) || (tx == TX_RB)) {
	    utx.begin();
	}
	ARemote a1 = ahome.findByPrimaryKey("a1");
	BRemote b1 = bhome.findByPrimaryKey("b1");
	ARemote a2 = ahome.findByPrimaryKey("a2");
	BRemote b2 = bhome.findByPrimaryKey("b2");
	ARemote a3 = ahome.findByPrimaryKey("a3");
	BRemote b3 = bhome.findByPrimaryKey("b3");
	// remove the bean
	bhome.remove("b1");
	if (tx == TX_CALL) {
	    utx.commit();
	} else if (tx == TX_RB) {
	    utx.rollback();
	}
	if (tx != TX_RB) {
	    // Verify expected result
	    assertNull("Bad coherence of relation : null expected for a1.retreiveB() found :"+a1.retrieveB(), a1.retrieveB());	
	    boolean not_found=false;
	    try {
		b1 = bhome.findByPrimaryKey("b1");
	    } catch (ObjectNotFoundException e) {
		not_found=true;
	    }
	    assertTrue("B is not removed",not_found);
	    // undo
	    bhome.create("b1");
	    a1.assignB("b1");
	}     
	checkIsInitialState();
    }

    public void testCohRemoveBTxNo() throws Exception {
	_testCohRemoveB(TX_NO);
    }
    public void testCohRemoveBTxCall() throws Exception {
	_testCohRemoveB(TX_CALL);
    }
    public void testCohRemoveBTxRb() throws Exception {
	_testCohRemoveB(TX_RB);
    }

    /**
     * test coherence relation one to one bidirectionnel, 
     * B1.remove=>B1 removed && A1.retreiveB()==null
     * Same as  _testCohRemoveB except that the called remove method is on the bean
     * instead of the home.
     */
    public void _testCohBeanRemoveB(int tx) throws Exception {

	if (tx == TX_CONT) {
	    // The transaction attribute of the remove method is TX_SUPPORT,
	    // so the transaction cannot be initiate by the container
	    fail("Transaction cannot be initiate by the container for this test");
	}
   	       
	if ((tx == TX_CALL) || (tx == TX_RB)) {
	    utx.begin();
	}
	ARemote a1 = ahome.findByPrimaryKey("a1");
	BRemote b1 = bhome.findByPrimaryKey("b1");
	ARemote a2 = ahome.findByPrimaryKey("a2");
	BRemote b2 = bhome.findByPrimaryKey("b2");
	ARemote a3 = ahome.findByPrimaryKey("a3");
	BRemote b3 = bhome.findByPrimaryKey("b3");
	// change the relation
	b1.remove();
	if (tx == TX_CALL) {
	    utx.commit();
	} else if (tx == TX_RB) {
	    utx.rollback();
	}
	if (tx != TX_RB) {
	    // Verify expected result
	    assertNull("Bad coherence of relation : null expected for a1.retreiveB() found :"+a1.retrieveB(), a1.retrieveB());	
	    boolean not_found=false;
	    try {
		b1 = bhome.findByPrimaryKey("b1");
	    } catch (ObjectNotFoundException e) {
		not_found=true;
	    }
	    assertTrue("B is not removed",not_found);
	    // undo
	    bhome.create("b1");
	    a1.assignB("b1");
	}     
	checkIsInitialState();
    }

    public void testCohBeanRemoveBTxNo() throws Exception {
	_testCohBeanRemoveB(TX_NO);
    }
    public void testCohBeanRemoveBTxCall() throws Exception {
	_testCohBeanRemoveB(TX_CALL);
    }
    public void testCohBeanRemoveBTxRb() throws Exception {
	_testCohBeanRemoveB(TX_RB);
    }

    public static Test suite() {
	return new TestSuite(F_Relation_oobEC2.class);
    }

    public static void main (String args[]) {
	String testtorun = null;
	// Get args
	for (int argn = 0; argn < args.length; argn++) {
	    String s_arg = args[argn];
	    Integer i_arg;
	    if (s_arg.equals("-n")) {
		testtorun = args[++argn];
	    }
	}
	if (testtorun == null) {
	    junit.textui.TestRunner.run(suite());
	} else {
	    junit.textui.TestRunner.run(new F_Relation_oobEC2(testtorun));
	}
    }
}

